// Generated by CoffeeScript 1.7.1
(function() {
  var Converter, debug, fs, path, requires,
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; },
    __slice = [].slice;

  fs = require('fs');

  path = require('path');

  requires = require('requires');

  debug = require('debug')('component2duo');

  Converter = (function() {
    var STYLE;

    STYLE = [0, 1];

    Converter.ABSOLUTE = STYLE[0];

    Converter.RELATIVE = STYLE[1];

    function Converter(rootPath, style, opt) {
      var _ref;
      this.rootPath = rootPath;
      this.style = style != null ? style : ABSOLUTE;
      if (opt == null) {
        opt = {};
      }
      if (_ref = this.style, __indexOf.call(STYLE, _ref) < 0) {
        throw new Error('style should either be "absolute" or "relative"');
      }
      this.simulate = opt.simulate || false;
      this.debugPrint = opt.debug || false;
      debug("debug: " + this.debugPrint + ", simulate: " + this.simulate);
      if (this.rootPath[0] !== '/') {
        this.rootPath = path.join(process.cwd(), this.rootPath);
      }
      debug("root path: " + this.rootPath);
      this.rootDir = path.dirname(this.rootPath);
      this.localsMap = {};
      this.localManifest = opt.localManifest || 'component.json';
      if (!((this.rootPath != null) || this.rootPath === '')) {
        throw new Error('no root path was given');
      }
    }

    Converter.prototype.start = function() {
      var absPath, componentPath, filePath, fullPath, localName, newContent, r, script, scripts, simulateContainer, _i, _len, _ref, _ref1;
      fullPath = path.join(this.rootPath);
      debug("starting at " + fullPath);
      r = require(fullPath);
      debug("root component.json - ok");
      this.lookupPath = r.paths || r.path;
      if (this.lookupPath.length === 0) {
        throw new Error('need a lookup path for locals');
      }
      if (this.lookupPath.length > 1) {
        throw new Error('multiple paths not implemented');
      }
      this.lookupPath = this.lookupPath[0];
      this.rootLocals = r.locals || r.local;
      debug("traversing locals for paths: " + this.lookupPath);
      this.traverseLocals(this.rootLocals);
      debug("finished traversing");
      debug("map:", this.localsMap);
      simulateContainer = {};
      _ref = this.localsMap;
      for (localName in _ref) {
        _ref1 = _ref[localName], absPath = _ref1.absPath, scripts = _ref1.scripts;
        debug("analyze " + localName);
        for (_i = 0, _len = scripts.length; _i < _len; _i++) {
          script = scripts[_i];
          debug("file " + script);
          componentPath = path.join(absPath, script);
          filePath = path.join(this.rootDir, componentPath);
          newContent = this.rewriteRequire(componentPath, filePath);
          if (this.simulate) {
            if (simulateContainer[localName] == null) {
              simulateContainer[localName] = {};
            }
            simulateContainer[localName][script] = newContent;
          } else {
            fs.writeFileSync(filePath, newContent, 'utf8');
          }
        }
      }
      return simulateContainer;
    };

    Converter.prototype.traverseLocals = function(locals) {
      var local, manifest, manifestPath, transLocals, _i, _len, _results;
      _results = [];
      for (_i = 0, _len = locals.length; _i < _len; _i++) {
        local = locals[_i];
        if (this.localsMap[local]) {
          continue;
        }
        debug("traverse '" + local + "'");
        manifestPath = path.join(this.lookupPath, local, this.localManifest);
        manifest = require(path.join(this.rootDir, manifestPath));
        debug("found manifest");
        if (manifest.paths != null) {
          console.log("'" + local + "': ignore paths: " + manifest.paths);
        }
        this.localsMap[local] = {
          scripts: manifest.scripts,
          absPath: path.dirname(manifestPath)
        };
        transLocals = manifest.locals || manifest.local || [];
        debug("locals: " + transLocals);
        if (transLocals.length > 0) {
          _results.push(this.traverseLocals(transLocals));
        } else {
          _results.push(void 0);
        }
      }
      return _results;
    };

    Converter.prototype.rewriteRequire = function(scriptPath, fullPath) {
      var content, currentDir, newContent;
      currentDir = path.dirname(scriptPath);
      content = fs.readFileSync(fullPath, 'utf8');
      newContent = requires(content, (function(_this) {
        return function(require) {
          var absPath, debugStr, newRequire, quote, requireRoot, rest, _ref;
          quote = require.string.match(/"/) ? '"' : "'";
          _ref = require.path.split('/'), requireRoot = _ref[0], rest = 2 <= _ref.length ? __slice.call(_ref, 1) : [];
          if (requireRoot === '') {
            throw new Error("cannot handle require path: " + require.path);
          }
          if (requireRoot === '.' || requireRoot === '..') {
            debug("ignore relative require path: '" + require.path + "'");
          } else if (_this.localsMap[requireRoot] != null) {
            absPath = _this.localsMap[requireRoot].absPath;
            if (_this.style === STYLE[0]) {
              newRequire = '/' + absPath;
              if (rest.length > 0) {
                newRequire += '/' + rest.join('/');
              }
            } else {
              newRequire = path.relative(currentDir, absPath);
              if (rest.length > 0) {
                newRequire += '/' + rest.join('/');
              }
            }
            debugStr = "" + scriptPath + ": '" + require.path + "' -> '" + newRequire + "'";
            debug(debugStr);
            if (_this.debugPrint) {
              console.log(debugStr);
            }
            return "require(" + quote + newRequire + quote + ")";
          } else {
            debug("ignore remote require: '" + requireRoot + "'");
          }
          return "require(" + quote + require.path + quote + ")";
        };
      })(this));
      return newContent;
    };

    return Converter;

  })();

  module.exports = Converter;

}).call(this);
